/***************************************************************************
 *                                                                         *
 *  Copyright 1995 Symbios Logic Incorporated.  All rights reserved.       *
 *                                                                         *
 * This software was written by Symbios Logic Inc. to develop and test new *
 * products.  Symbios Logic assumes no liability for its use. This software*
 * is released to the public domain to illustrate certain programming      *
 * techniques for testing the 53c8xx chips                                 *
 *                                                                         *
 ***************************************************************************/

/*           Name:  8XX_TOOL.H
 *          Title:  Validation test code 8XX_tool.c header
 *      $Workfile:  $
 *      $Revision:  $
 *       $Modtime:  $
 *     Programmer:  Symbios Logic
 *  Creation Date:  11/29/95
 *
 *  Version History
 *  ---------------
 *
 *    Date    Who?  Description
 *  --------  ----  -------------------------------------------------------
#BeginRevision
 *  03/06/96  SYM   - Added #defines for Device features, Burst Sizes,
 *						  Device type.
 *						  - Converted iowriteDSP to a macro and moved it into this
 *						  file.
#EndRevision
 *
 *  11/29/95  SYM   Initial Revision.
 *
#BeginDescription
 *
 *  This file contains structures and #defines used by the 8XX_TOOL.C source
 *  file.
 *
#EndDescription
 *
 *-------------------------------------------------------------------------
 *
 *  $Header: $
 *
 */

#include "valtypes.h"

/* Return values from functions */
/* Returned by any function in C8XX_TOOL.C which successfully completed */
#define         C8XX_GOOD                               0x00
/* Returned by PollISTAT to indicate no chip interrupt was detected in the 
   allotted time */
#define         C8XX_POLLTO                             0x82
/* Returned by CopyToScRam to indicate the script copy to Script RAM failed */
#define         C8XX_RAMCOPYFAILURE     0x83
/* Returned by WriteScRam and WriteLongScRam to indicate the write to the 
   script RAM failed */
#define         C8XX_RAMWRITEFAILURE    0x84
/* Returned by ReadScRam and ReadLongScRam to indicate the Read of the 
   script RAM failed */
#define         C8XX_RAMREADFAILURE     0x85

/* 53C8XX Interrupt bits */
/* DMA interrupt pending (ISTAT) */
#define         C8XX_INT_DIP    0x01
/* SCSI interrupt pending (ISTAT) */
#define         C8XX_INT_SIP    0x02
/* Master parity error (DSTAT) */
#define         C8XX_INT_MDPE   0x40
/* Bus Fault (DSTAT) */
#define         C8XX_INT_BF     0x20
/* Scripts aborted (DSTAT) */
#define         C8XX_INT_ABRT   0x10
/* Script Step interrupt (DSTAT) */
#define         C8XX_INT_SSI    0x08
/* Script Interrupt instruction (DSTAT) */
#define         C8XX_INT_SIR    0x04
/* Script Illegal instruction detected (DSTAT) */
#define                  C8XX_INT_IID     0x01
/* Phase Mismatch (SIST) */
#define         C8XX_INT_MA     0x0080
/* Function Complete (SIST) */
#define         C8XX_INT_CMP    0x0040
/* Selected (SIST) */
#define         C8XX_INT_SEL    0x0020
/* Reselected (SIST) */
#define         C8XX_INT_RSL    0x0010
/* SCSI Gross error (SIST) */
#define         C8XX_INT_SGE    0x0008
/* Unexpected disconnect (SIST) */
#define         C8XX_INT_UDC    0x0004
/* SCSI Reset received (SIST) */
#define         C8XX_INT_RST    0x0002
/* SCSI Parity error (SIST) */
#define         C8XX_INT_PAR    0x0001
/* SCSI Selection time-out (SIST) */
#define         C8XX_INT_STO    0x0400
/* General purpose timer int (SIST) */
#define         C8XX_INT_GEN    0x0200
/* Handshake to Handshake int (SIST) */
#define         C8XX_INT_HTH    0x0100

/* 53C8XX Device listing for the c8xxdev_record.device field below */
#define C8XX_53C810    0
#define C8XX_53C810A   1
#define C8XX_53C810ALV 2
#define C8XX_53C815    3
#define C8XX_53C825    4
#define C8XX_53C825A   5
#define C8XX_53C860    6
#define C8XX_53C860LV  7
#define C8XX_53C875    8
#define C8XX_53C885    9
#define C8XX_53C895    10
#define C8XX_53C896    11
#define C8XX_UNKNOWN_DEVICE 256

/* Feature masks for c8xxdev_record below */
#define C8XX_WIDE_SCSI 0x00000001l
#define C8XX_ROM_IF    0x00000002l
#define C8XX_DIFF_SCSI 0x00000004l
#define C8XX_BRST_OP   0x00000008l
#define C8XX_PREFETCH  0x00000010l
#define C8XX_SRAM      0x00000020l
#define C8XX_CACHE     0x00000040l
#define C8XX_SSAID     0x00000080l
#define C8XX_EX_TIMER  0x00000100l
#define C8XX_EX_SLPAR  0x00000200l
#define C8XX_EN_RMOVE  0x00000400l
#define C8XX_IRQ_DIS   0x00000800l
#define C8XX_LD_STR    0x00001000l
#define C8XX_FAST20    0x00002000l
#define C8XX_32SCRATCH 0x00004000l
#define C8XX_CLK_DBLR  0x00008000l
#define C8XX_FAST40    0x00010000l
#define C8XX_64BITPCI  0x00020000l

/* Burst Sizes for Feature sets below */
#define C8XX_BURST_2   0
#define C8XX_BURST_4   1
#define C8XX_BURST_8   2
#define C8XX_BURST_16  3
#define C8XX_BURST_32  4
#define C8XX_BURST_64  5
#define C8XX_BURST_128 6

/* Feature info for all devices for c8xxdev_record below */
#define C8XX_53C810_FEAT                0x00000000l
#define C8XX_53C810_FIFO_A              64
#define C8XX_53C810_FIFO_B              0
#define C8XX_53C810_BRST_A              C8XX_BURST_16
#define C8XX_53C810_BRST_B              0
#define C8XX_53C810_OFFSET              8
#define C8XX_53C810A_FEAT               0x000018D8l
#define C8XX_53C810A_FIFO_A     80
#define C8XX_53C810A_FIFO_B     0
#define C8XX_53C810A_BRST_A     C8XX_BURST_16
#define C8XX_53C810A_BRST_B     0
#define C8XX_53C810A_OFFSET     8
#define C8XX_53C810ALV_FEAT     0x000018D8l
#define C8XX_53C810ALV_FIFO_A   80
#define C8XX_53C810ALV_FIFO_B   0
#define C8XX_53C810ALV_BRST_A   C8XX_BURST_16
#define C8XX_53C810ALV_BRST_B   0
#define C8XX_53C810ALV_OFFSET   8
#define C8XX_53C815_FEAT                0x0000000Al
#define C8XX_53C815_FIFO_A              64
#define C8XX_53C815_FIFO_B              0
#define C8XX_53C815_BRST_A              C8XX_BURST_16
#define C8XX_53C815_BRST_B              0
#define C8XX_53C815_OFFSET              8
#define C8XX_53C825_FEAT                0x0000000Fl
#define C8XX_53C825_FIFO_A              64
#define C8XX_53C825_FIFO_B              0
#define C8XX_53C825_BRST_A              C8XX_BURST_16
#define C8XX_53C825_BRST_B              0
#define C8XX_53C825_OFFSET              8
#define C8XX_53C825A_FEAT               0x00005FFFl
#define C8XX_53C825A_FIFO_A     88
#define C8XX_53C825A_FIFO_B     536
#define C8XX_53C825A_BRST_A     C8XX_BURST_16
#define C8XX_53C825A_BRST_B     C8XX_BURST_128
#define C8XX_53C825A_OFFSET     16
#define C8XX_53C860_FEAT                0x000038D8l
#define C8XX_53C860_FIFO_A              80
#define C8XX_53C860_FIFO_B              0
#define C8XX_53C860_BRST_A              C8XX_BURST_16
#define C8XX_53C860_BRST_B              0
#define C8XX_53C860_OFFSET              8
#define C8XX_53C860LV_FEAT              0x000038D8l
#define C8XX_53C860LV_FIFO_A    80
#define C8XX_53C860LV_FIFO_B    0
#define C8XX_53C860LV_BRST_A    C8XX_BURST_16
#define C8XX_53C860LV_BRST_B    0
#define C8XX_53C860LV_OFFSET    8
#define C8XX_53C875_FEAT                0x00007FFFl
#define C8XX_53C875_FIFO_A              88
#define C8XX_53C875_FIFO_B              536
#define C8XX_53C875_BRST_A              C8XX_BURST_16
#define C8XX_53C875_BRST_B              C8XX_BURST_128
#define C8XX_53C875_OFFSET              16
#define C8XX_53C885_FEAT                0x0000FFFBl
#define C8XX_53C885_FIFO_A              88
#define C8XX_53C885_FIFO_B              536
#define C8XX_53C885_BRST_A              C8XX_BURST_16
#define C8XX_53C885_BRST_B              C8XX_BURST_128
#define C8XX_53C885_OFFSET              16
#define C8XX_53C895_FEAT                0x0001FFFFl
#define C8XX_53C895_FIFO_A    112
#define C8XX_53C895_FIFO_B              816
#define C8XX_53C895_BRST_A              C8XX_BURST_16
#define C8XX_53C895_BRST_B              C8XX_BURST_128
#define C8XX_53C895_OFFSET              31
#define C8XX_53C896_FEAT                0x0003FFFFl
#define C8XX_53C896_FIFO_A              112
#define C8XX_53C896_FIFO_B              816
#define C8XX_53C896_BRST_A              C8XX_BURST_16
#define C8XX_53C896_BRST_B              C8XX_BURST_128
#define C8XX_53C896_OFFSET              31

/* 53C8XX register set */
#define SCNTL0  0x00
#define  SCNTL1  0x01
#define  SCNTL2  0x02
#define  SCNTL3  0X03
#define  SCID  0x04
#define  SXFER  0x05
#define  SDID  0x06
#define  GPREG  0x07
#define  SFBR  0x08
#define  SOCL  0x09
#define  SSID  0x0A
#define  SBCL  0x0B
#define  DSTAT  0x0C
#define  SSTAT0  0x0D
#define  SSTAT1  0x0E
#define  SSTAT2  0x0F
#define  DSA  0x10
#define  ISTAT  0x14
#define  CTEST0  0x18
#define  CTEST1  0x19
#define  CTEST2  0x1A
#define  CTEST3  0x1B
#define  TEMP  0x1C
#define  DFIFO  0x20
#define  CTEST4  0x21
#define  CTEST5  0x22
#define  CTEST6  0x23
#define  DBC  0x24
#define  DCMD  0x27
#define  DNAD  0x28
#define  DSP  0x2C
#define  DSPS  0x30
#define  SCRATCHA  0x34
#define  DMODE  0x38
#define  DIEN  0x39
#define  SBR  0x3A
#define  DCNTL  0x3B
#define  ADDER  0x3C
#define  SIEN0  0x40
#define  SIEN1  0x41
#define  SIST0  0x42
#define  SIST1  0x43
#define  SLPAR  0x44
#define  SWIDE  0X45
#define  MACNTL  0x46
#define  GPCNTL  0x47
#define  STIME0  0x48
#define  STIME1  0x49
#define  RESPID0  0x4A
#define  RESPID1  0x4B
#define  STEST0  0x4C
#define  STEST1  0x4D
#define  STEST2  0x4E
#define  STEST3  0x4F
#define  SIDL  0x50
#define  SODL  0x54
#define  SODL1  0x55
#define  SBDL  0x58
#define  SCRATCHB  0x5C
#define  SCRATCHC  0x60
#define  SCRATCHD  0x64
#define  SCRATCHE  0x68
#define  SCRATCHF  0x6C
#define  SCRATCHG  0x70
#define  SCRATCHH  0x74
#define  SCRATCHI  0x78
#define  SCRATCHJ  0x7C

/* ti_entry - Used for creating Script table indirect entries. */

typedef struct {
	VULONG count;
	VULONG address;
} ti_entry;

/* field definitions for ti_entry:
	count - Either a byte count in the Block Move Table Indirect entry
		or Selection information in a Select Table Indirect entry.
		The select information format is as follows
		(0x) SCNTL3:SDID:SXFER:00.
	address - This is the destination/source data address in a Block
		  Move Table Indirect entry.  This field is not used in a
		  Select Table Indirect entry and must be set to 0.
*/

/* c8xxdev_record - Used to store device feature information */

typedef struct {
	VINT device;
	VINT fifo_size_a;
	VINT fifo_size_b;
	VINT max_burst_a;
	VINT max_burst_b;
	VINT max_offset;
	VULONG features;
} c8xxdev_record;

/* field definitions for c8xxdev_record:
	device - This is the type of device being tested.  This value is #defined
				for all known devices above
	features - indicates the particular features that a 8xx device supports
		The bit definitions are as follows:
			Bit 0 - Device supports Wide SCSI data transfers
			Bit 1 - Device has a back end ROM interface
			Bit 2 - Device supports differential
			Bit 3 - Device supports burst op code fetch
			Bit 4 - Device supports instruction prefetch
			Bit 5 - Device has Scripts RAM
			Bit 6 - Device supports Cache Line Size and Cache Commands
			Bit 7 - Device supports SSAID register
			Bit 8 - Device supports expanded timers
			Bit 9 - Device supports expanded SLPAR register
			Bit 10 - Device supports enhanced register move script instruction
			Bit 11 - Device supports IRQ disable
			Bit 12 - Device supports LOAD/STORE script instructions
			Bit 13 - Device supports Fast-20 transfers
			Bit 14 - Device supports additional 32 Scratch registers
			Bit 15 - Device supports Clock Doubler
			Bit 16 - Device supports Fast-40 transfers
		The remaining bit definitions are reserved for future use.
	fifo_size_a - This is the power up fifo size of the device
	fifo_size_b - If the device supports a larger programmable fifo size
					  This will contain that size.  It will be zero if the device
					  does not support another fifo size;
	max_burst_a - This is the maximum burst size that the device will support
					  given that the fifo size is at fifo_size_a above
	max_burst_b - This is the maximum burst size that the device will support
					  given that the fifo size is at fifo_size_b above.  If the
					  device does not support fifo_size_b then this will be zero
	max_offset - This is the maximum synchronous offset the device will support
*/

/* macro functions */
/******************************************************************************
 Function: iowriteDSP
	Purpose: To write to DSP register to start script execution
	Input: The IO Base address of the 53C8XX part
			 The value to be written to the DSP
	Output: Nothing
	Assumptions: SCSI Scripts are currently not executing
	Restrictions:  Do not call this macro while scripts are executing
	Other functions called: IOWrite32 to write the DSP

******************************************************************************/
#define iowriteDSP(IO_Base, value) IOWrite32(IO_Base+DSP, value)

/* function prototypes */
extern VINT init_8XX(pcidev_record *PCIDevice);
extern VULONG *align_script(VULONG *original_script, VUINT size);
extern VINT PollISTAT(VULONG IO_Base, VULONG *vector, VUBYTE *dstat,
					VUBYTE *istat, VUINT *sist, VUINT timeout);
extern VINT CopyToScRam (pcidev_record *PCIDevice, VULONG far *script, VUINT size);
extern VINT WriteScRam(pcidev_record *PCIDevice, VUBYTE byte, VINT offset);
extern VINT ReadScRam(pcidev_record *PCIDevice, VUBYTE *byte, VINT offset);
extern VINT WriteLongScRam(pcidev_record *PCIDevice, VULONG dword, VINT offset);
extern VINT ReadLongScRam(pcidev_record *PCIDevice, VULONG *dword, VINT offset);
extern VINT GenRandSG(VUBYTE far *buf, VUINT size, ti_entry *sg_list, VINT num_entries);
extern VINT C8XX_GetChipType (pcidev_record *PCIDevice, c8xxdev_record *C8XXDevice);
extern void C8XX_SetBurstSize(pcidev_record *PCIDevice, VINT burst);


