/***************************************************************************
 *                                                                         *
 *  Copyright 1995 Symbios Logic Incorporated.  All rights reserved.       *
 *                                                                         *
 *  This file is confidential and a trade secret of Symbios Logic.  The    *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of Symbios Logic Incorporated.                         *
 *                                                                         *
 ***************************************************************************/

/*           Name:  PCI.H
 *          Title:  Validation test code pci.c header
 *      $Workfile:  $
 *      $Revision:  $
 *       $Modtime:  $
 *     Programmer:  Symbios Logic
 *  Creation Date:  11/29/95
 *
 *  Version History
 *  ---------------
 *
 *    Date    Who?  Description
 *  --------  ----  -------------------------------------------------------
#BeginRevision
 *  11/29/95  SYM   Initial Revision.
#EndRevision
 *
#BeginDescription
 *
 *  This file contains structures and #defines used by the PCI.C source
 *  file.
 *  The core of this code was originally designed by Richard Soloman.
 *
#EndDescription
 *
 *-------------------------------------------------------------------------
 *
 *  $Header: $
 *
 */

#include "valtypes.h"

/* System Bios Interrupt Vector (0x1A) */
#define         PCI_BIOS_INT                    0x1A
/* Function for PCI system bios call (0xB1) */
#define         PCI_FUNCTION_ID         0xB1
/* Symbios Logic's vendor ID (0x1000) */
#define         PCI_SYM_VENDOR_ID               0x1000
/* Used to make PCI bios present call (0x01) */
#define         PCI_BIOS_PRESENT                                0x01
/* Used to make PCI find device call (0x02) */
#define         PCI_FIND_DEVICE                         0x02
/* Used to make PCI read config dword call (0x0A) */
#define         PCI_READ_CONFIG_DWORD           0x0A
/* Used to make PCI write config dword call (0x0D) */
#define         PCI_WRITE_CONFIG_DWORD          0x0D
/* Returned by FindSYMDevice indicating that the requested PCI device was found */
#define         PCI_DEVICE_FOUND                0x01
/* Returned by FindSYMDevice indicating that the requested PCI device was not found */
#define         PCI_NO_DEVICE_FOUND     0x00
/* Returned by PCI_GetPCIBIOSVersion to indicate the PCI bios version is 1.X */
#define         PCI_BIOS_REV_1X         0x10
/* Returned by PCI_GetPCIBIOSVersion to indicate the PCI bios version is 2.X */
#define         PCI_BIOS_REV_2X         0x20
/* Returned by PCI_GetPCIBIOSVersion to indicate the PCI bios version is unknown */
#define         PCI_UNKNOWN_BIOS                0xFF
/* Returned by PCI_GetPCIBIOSVersion to indicate that a PCI bios is not present in the system */
#define         PCI_NO_BIOS                             0x00

/* PCI configuration regs */
#define         PCI_CONFIG_REG_VENID    0x00    /* vendor id */
#define         PCI_CONFIG_REG_DEVID    0x00    /* device id */
#define         PCI_CONFIG_REG_CMD      0x04    /* command register */
#define         PCI_CONFIG_REG_STAT     0x04    /* status register */
#define         PCI_CONFIG_REG_REVID    0x08    /* revision id */
#define         PCI_CONFIG_REG_CLASS    0x08    /* class code */
#define         PCI_CONFIG_REG_CLS      0x0C    /* cache line size */
#define         PCI_CONFIG_REG_LAT      0x0C    /* latency timer */
#define         PCI_CONFIG_REG_HDRT     0x0C    /* header type */
#define         PCI_CONFIG_REG_BIST     0x0C    /* built in self test reg */
#define         PCI_CONFIG_REG_CIS      0x28    /* cardbus CIS pointer */
#define         PCI_CONFIG_REG_SUBV     0x2C    /* Subsystem vendor id */
#define         PCI_CONFIG_REG_SUBID    0x2C    /* subsystem id */
#define         PCI_CONFIG_REG_ROM      0x30    /* Rom Base address */
#define         PCI_CONFIG_REG_INTL     0x3C    /* interrupt line */
#define         PCI_CONFIG_REG_INTP     0x3C    /* interrupt pin */
#define         PCI_CONFIG_REG_MGNT     0x3C    /* min grant */
#define         PCI_CONFIG_REG_MLAT     0x3C    /* max lat */


/* Structures returned and required by functions */

/* pcidev_record -  returned by FindSYMDevice 
   type device record required by GetConfigRegister and SetConfigRegister */

typedef struct {
			VUINT  bus_num;
			VUINT  device_num;
			VUINT  function;
			VUINT  device_id;
			VUINT  device_index;
			VUBYTE rev_id;
			VUBYTE CLS;
			VUBYTE latency;
			VUINT  command;
			VULONG base_addr1;
			VULONG base_addr2;
			VULONG base_addr3;
			VULONG base_addr4;
			VULONG base_addr5;
			VULONG base_addr6;
			VULONG CIS;
			VUINT  sub_id;
			VUINT  sub_ven_id;
			VULONG ROM_base_addr;
			VUBYTE intl;
			VUBYTE intp;

} pcidev_record;

/* pcidev_record field descriptions:
	*bus_num - the bus number the PCI device resides on
	*device_num - the PCI device number of the PCI device
	*function - the function number of the PCI device
	*device_id - the device ID of the PCI device
	*device_index - the index number of the PCI device
	*rev_id - the revision id of the PCI device
	*CLS - the cache line size register of the PCI device
	*latency - the latency timer of the PCI device
	*command - the command register of the PCI device
	base_addr1 - BAR1 (8XX Register Memory base)
	*base_addr2 - BAR2 (8XX Register IO base)
	*base_addr3 - BAR3 (8XX Script RAM base 825A/875 only)
	base_addr4 - BAR4
	base_addr5 - BAR5
	base_addr6 - BAR6
	CIS - Cardbus CIS pointer
	sub_id - Subsystem ID of the PCI Device
	sub_ven_id - Subsystem vendor ID of the PCI device
	*ROM_base_addr - ROM base address of the PCI device
	*intl - Interrupt line of the PCI device
	*intp - Interrupt pin of the PCI device

	* - These variables are currently filled in by the Core code when 
	initializing PCI devices. Note that if a PCI config register is 
	changed then the corresponding information in this data structure 
	should be changed also.
*/

/* pci_bios_info - structure returned by PCI_GetPCIBIOSVersion.  
   Contains information about PCI system bios. */

typedef struct {
			VUBYTE access_mech;
			VUINT interface_lvl;
			VUBYTE last_bus;
} pci_bios_info;

/* pci_bios_info field descriptions:
	access_mech - Bit 0 set, Hardware mech #1 supported/ Bit 1 set, 
		      Hardware mech #2 supported
	interface_lvl - Upper byte = major version, 
			lower byte = minor version
	last_bus - Last PCI bus number in system
*/

/* function prototypes */
extern VINT PCI_FindSYMDevice(pcidev_record *PCIDevice);
extern void PCI_SetConfigRegister(pcidev_record *PCIDevice, VUINT offset,
										VULONG value);
extern VULONG PCI_GetConfigRegister(pcidev_record *PCIDevice,
				    VUINT offset);
extern VUINT  PCI_GetPCIBIOSVersion(pci_bios_info *pcibios);
extern void PCI_check_pci_bios(pci_bios_info *pcibios);




