; ***************************************************************************
; *                                                                         *
; *  Copyright 1995 Symbios Logic Incorporated.  All rights reserved.       *
; *                                                                         *
; * This software was written by Symbios Logic Inc. to develop and test new *
; * products.  Symbios Logic assumes no liability for its use. This software*
; * is released to the public domain to illustrate certain programming      *
; * techniques for testing the 53c8xx chips                                 *
; *                                                                         *
; ***************************************************************************
;
;                         Name:  RW_SCR.SS
;                        Title:  Validation test code 53C8XX Read/Write scripts
;               $Workfile:  $
;               $Revision:  $
;                $Modtime:  $
;         Programmer:  Symbios Logic
; Creation Date:  11/29/95
;
;  Version History
;  ---------------
;
;    Date    Who?  Description
;  --------  ----  -------------------------------------------------------
; #BeginRevision
;  03/06/96  SYM   - Removed Not_Disconnect interrupt vector, Msg_reject
;						 extern buffer, and Get_Msg_Reject Entry point
;						 - Added code to set a status bit in DWT(SBR) when all
;						 data was transferred.
;						 - Added a general phase handler to replace all the specific
;						 code to handle certain messaging and status situations.
;						 - Added Handle Phase entry point
; #EndRevision
;
;  11/29/95  SYM   Initial Revision.
;
; #BeginDescription
;
;   This file contains the SCSI Scripts used to execute single threaded
;   Read/Write IO's.  The scripts support Disconnect/Reselect, Tagged
;   queuing, Scatter Gather, synchronous, and wide.
;
; #EndDescription
;
; -------------------------------------------------------------------------
;
;   $Header: $
;


ARCH 825                ; This code is compatible with all 8XX devices

ABSOLUTE RW_Complete_int = 0x00
ABSOLUTE RW_Not_Tag_Msg = 0x02
ABSOLUTE RW_Unexpected_Message = 0x03
ABSOLUTE RW_Unhandled_Phase_in_bitbucket = 0x04
ABSOLUTE RW_Need_More_SG = 0x06
ABSOLUTE RW_SWIDE_byte_moved = 0x07
ABSOLUTE RW_NEED_MORE_SG_ENTRIES = 0x01
ABSOLUTE RW_ALL_DATA_TRANSFERRED = 0x02

EXTERN RW_resel_msg_buf
EXTERN RW_resel_tag_buf1
EXTERN RW_resel_tag_buf2
EXTERN RW_Last_di_byte_buf
EXTERN RW_Bitbucket_buf

TABLE table0 \
		  rw_scsi_id=  ID{0x33, 0x00, 0x00, 0x00}, \
		  rw_id_buf= {0xC0}, \
		  rw_cmd_buf= {0x08, 0x00, 0x00, 0x00, 0x08, 0x00}, \
		  rw_stat_buf= ??, \
		  rw_msgi_buf= ??,\
		  rw_data_buf1= 512{??}, \
		  rw_data_buf2= 512{??}, \
		  rw_data_buf3= 512{??}, \
		  rw_data_buf4= 512{??}, \
		  rw_data_buf5= 512{??}, \
		  rw_data_buf6= 512{??}, \
		  rw_data_buf7= 512{??}, \
		  rw_data_buf8= 512{??}, \
		  rw_data_buf9= 512{??}, \
		  rw_data_buf10= 512{??}, \
		  rw_data_buf11= 512{??}, \
		  rw_data_buf12= 512{??}, \
		  rw_data_buf13= 512{??}, \
		  rw_data_buf14= 512{??}, \
		  rw_data_buf15= 512{??}, \
		  rw_data_buf16= 512{??}


ENTRY RW_Offset_patch_do            ; label used to enable patching of a
												; relative jump for scatter gather operations
ENTRY RW_Offset_patch_di            ; same as above but for data in
ENTRY RW_Start                      ; IO Start entry
ENTRY RW_Handle_Phase               ; This handles most general purpose phase stuff
ENTRY RW_Bitbucket
ENTRY RW_Move_swide_byte

PROC RW_SCRIPT:

RW_Wait_Resel:
WAIT RESELECT REL(0)                    ; handle reselect from here

MOVE 1, RW_resel_msg_buf, WHEN MSG_IN
CLEAR ACK
JUMP REL(Get_Tag), WHEN MSG_IN          ; if still msg in then queue tag
													 ; is coming (hopefully)

SELECT FROM rw_scsi_id, REL(Move_Data)  ; This is a quick way to update
													 ; synch and wide info for this device
													 ; The inst. will always take the alt_jump

Get_Tag:
MOVE 1, RW_resel_tag_buf1, WHEN MSG_IN      ; Get tag message, we hope
INT RW_Not_Tag_Msg, IF NOT 0x20 AND MASK 0x03 ; Interrupt if not tag message
CLEAR ACK
MOVE 1, RW_resel_tag_buf2, WHEN MSG_IN      ; Get actual tag
CLEAR ACK

SELECT FROM rw_scsi_id, REL(Move_Data)  ; This is a quick way to update
													 ; sync and wide info for this device
													 ; The inst. will always take the alt_jump

Move_Data:
JUMP REL(RW_Offset_patch_di), WHEN DATA_IN

JUMP REL(RW_Handle_Phase) WHEN NOT DATA_OUT     ; figure out what's up
													 ; after all data has been sent
													 ; and then resel. with stat and mi

; *** Script move data out ENTRY
RW_Offset_patch_do:
JUMP REL(Data_Out_xfer)                 ; Relative offset will be changed so
													 ; that we jump into the proper place in
													 ; the scatter gather list
Data_Out_xfer:
CHMOV FROM rw_data_buf1, WHEN DATA_OUT  ; 16 moves to support Scatter Gather
CHMOV FROM rw_data_buf2, WHEN DATA_OUT
CHMOV FROM rw_data_buf3, WHEN DATA_OUT
CHMOV FROM rw_data_buf4, WHEN DATA_OUT
CHMOV FROM rw_data_buf5, WHEN DATA_OUT
CHMOV FROM rw_data_buf6, WHEN DATA_OUT
CHMOV FROM rw_data_buf7, WHEN DATA_OUT
CHMOV FROM rw_data_buf8, WHEN DATA_OUT
CHMOV FROM rw_data_buf9, WHEN DATA_OUT
CHMOV FROM rw_data_buf10, WHEN DATA_OUT
CHMOV FROM rw_data_buf11, WHEN DATA_OUT
CHMOV FROM rw_data_buf12, WHEN DATA_OUT
CHMOV FROM rw_data_buf13, WHEN DATA_OUT
CHMOV FROM rw_data_buf14, WHEN DATA_OUT
CHMOV FROM rw_data_buf15, WHEN DATA_OUT
CHMOV FROM rw_data_buf16, WHEN DATA_OUT

; Check to see if we need more SG list entries
MOVE DWT & RW_NEED_MORE_SG_ENTRIES to SFBR
INT RW_Need_More_SG, if not 0

; If we are here then all the data was transferred
; so we set a flag to indicate that
MOVE DWT | RW_ALL_DATA_TRANSFERRED to DWT

JUMP REL(RW_Handle_Phase)

; *** Script move data ENTRY
RW_Offset_patch_di:
JUMP REL(Data_In_xfer)                  ; Relative offset will be changed so
													 ; that we jump into the proper place in
													 ; the scatter gather list
Data_In_xfer:
CHMOV FROM rw_data_buf1, WHEN DATA_IN   ; 16 moves to support Scatter Gather
CHMOV FROM rw_data_buf2, WHEN DATA_IN
CHMOV FROM rw_data_buf3, WHEN DATA_IN
CHMOV FROM rw_data_buf4, WHEN DATA_IN
CHMOV FROM rw_data_buf5, WHEN DATA_IN
CHMOV FROM rw_data_buf6, WHEN DATA_IN
CHMOV FROM rw_data_buf7, WHEN DATA_IN
CHMOV FROM rw_data_buf8, WHEN DATA_IN
CHMOV FROM rw_data_buf9, WHEN DATA_IN
CHMOV FROM rw_data_buf10, WHEN DATA_IN
CHMOV FROM rw_data_buf11, WHEN DATA_IN
CHMOV FROM rw_data_buf12, WHEN DATA_IN
CHMOV FROM rw_data_buf13, WHEN DATA_IN
CHMOV FROM rw_data_buf14, WHEN DATA_IN
CHMOV FROM rw_data_buf15, WHEN DATA_IN
CHMOV FROM rw_data_buf16, WHEN DATA_IN

; Check to see if we need more SG list entries
MOVE DWT & RW_NEED_MORE_SG_ENTRIES to SFBR
INT RW_Need_More_SG, if not 0

; If we are here then all the data was transferred
; so we set a flag to indicate that
MOVE DWT | RW_ALL_DATA_TRANSFERRED to DWT

JUMP REL(RW_Handle_Phase)

; *** Main Script IO ENTRY
RW_Start:
SELECT ATN FROM rw_scsi_id, REL(0)      ; single threaded script so
													 ; we won't be reselected

JUMP REL(Send_Cmd), WHEN NOT MSG_OUT    ; could be SCSI-1 device
MOVE FROM rw_id_buf, WHEN MSG_OUT       ; send id message

Send_Cmd:
CLEAR ATN
MOVE FROM rw_cmd_buf, WHEN CMD

JUMP REL(Move_Data), WHEN DATA_IN
JUMP REL(Move_Data), WHEN DATA_OUT
JUMP REL(RW_Handle_Phase)

Get_Status:
MOVE FROM rw_stat_buf, WHEN STATUS
JUMP REL(RW_Handle_Phase), WHEN NOT MSG_IN

Get_Message:
CLEAR ACK
MOVE FROM rw_msgi_buf, WHEN MSG_IN
JUMP REL(Get_Message), IF 0x02
JUMP REL(IO_Disconnect), IF 0x04
JUMP REL(RW_Handle_Phase), IF 0x07
INT RW_Unexpected_Message, IF NOT 0x00

; Command Complete
; Expect a disconnect from the bus
MOVE SCNTL2 & 0x7F to SCNTL2
CLEAR ACK
WAIT DISCONNECT
INT RW_Complete_int

IO_Disconnect:
MOVE SCNTL2 & 0x7F to SCNTL2           ; Expect a disconnect from the bus
CLEAR ACK
WAIT DISCONNECT
MOVE CTEST3 | 0x04 to CTEST3           ; Clear the DMA fifo
MOVE STEST3 | 0x02 to STEST3           ; Clear the SCSI fifo
JUMP REL(RW_Wait_Resel)                                         ; Go up and wait for reselect

; *** Script move SWIDE byte ENTRY
RW_Move_swide_byte:
CHMOV 1, RW_Last_di_byte_buf, WHEN DATA_IN
INT RW_SWIDE_byte_moved

; *** Script Handle_Phase ENTRY
RW_Handle_Phase:
CLEAR ACK AND ATN
JUMP REL(Get_Message), WHEN MSG_IN
JUMP REL(Get_Status), WHEN STATUS
JUMP REL(Send_Cmd), WHEN CMD

; *** Script bitbucket ENTRY
; This routine tries to clear off any pending transfers from the
; SCSI bus so that the bus is free.
RW_Bitbucket:
CLEAR ACK                               ; gets rid of any dangling acks
JUMP REL(Move_MI), WHEN MSG_IN
JUMP REL(Move_DI), WHEN DATA_IN
JUMP REL(Move_Stat), WHEN STATUS
INT RW_Unhandled_Phase_in_bitbucket

Move_MI:
MOVE 1, RW_Bitbucket_buf, WHEN MSG_IN
CLEAR ACK
JUMP REL(RW_Bitbucket)

Move_DI:
MOVE 1, RW_Bitbucket_buf, WHEN DATA_IN
JUMP REL(RW_Bitbucket)

Move_Stat:
MOVE 1, RW_Bitbucket_buf, WHEN STATUS
JUMP REL(RW_Bitbucket)


